%{
#include <stdio.h>
#include <stdlib.h>

#include "lista.h"

#define YYDEBUG 1

extern int yylex();
extern FILE *yyin;
void yyerror(char * s){
	fprintf(stderr, "Sintaksna greska: \"%s\"\n",s);
	exit(1);
}

Lista* promenljive[26];
Lista* universe;

void Inicijalizacija(){
	int i;
	for(i=0; i<26;i++)
		promenljive[i]= NULL;

	universe = NULL;
}

void Deinicijalizacija(){
	int i;
	for(i=0; i<26;i++)
		ObrisiListu(&promenljive[i]);

	ObrisiListu(&universe);
}

void FormirajListu(Lista**lp, int begin, int end){
	int i;
	*lp =NULL;
	
	for (i =end; i>=begin; i--){
		Lista*tmp = novi(i);
		
		tmp ->sledeci=*lp;
		*lp=tmp;
	}
}

void Unija(Lista** dest, Lista* src){
	for(; src!= NULL; src= src->sledeci){
		Dodaj(dest,src->element);
	}
}

void Presek(Lista** dest, Lista* src){
	Lista* tmp;
	
	for(tmp =*dest ; tmp !=NULL; tmp=tmp->sledeci){
		if(Pronadji(src,tmp->element)==NULL)
			ObrisiElement(dest,tmp->element);
	}
}	

void Razlika(Lista**dest, Lista* src){
	Lista* tmp;
	
	for(tmp =*dest ; tmp !=NULL; tmp=tmp->sledeci){
		if(Pronadji(src,tmp->element)!=NULL)
			ObrisiElement(dest,tmp->element);
	}
}

int Podskup(Lista* manji , Lista*veci  ){
	for(; manji !=NULL; manji=manji->sledeci){
		if(Pronadji(veci, manji->element)==NULL)
			return 0;
	}
	return 1;
}
%}

%union{
	char ime;
	int i_val;
	Lista *elementi;
}

%nonassoc ':' '<'
%left U
%left P R
%right '~'

%token<ime> ID
%token<i_val> BROJ
%token PRINT TT CHECK CARD UNIVERSE

%type<elementi> skup lista

%start program
%%
program: program naredba		{}
			| 										{}
			;

naredba: ID '=' skup ';' {ObrisiListu(&promenljive[$1-'A']);
													promenljive[$1-'A'] = $3;}
	| PRINT skup ';'				{printf("{");
														Stampaj($2);
													printf("}\n");
													ObrisiListu(&$2);
	}
	|CHECK BROJ ':' skup ';' { if(Pronadji($4,$2) !=NULL)
															printf("True!\n");
													else printf("False!\n");
													ObrisiListu(&$4);	}
	| CHECK skup '<' skup ';' { if(Podskup($2,$4))
																printf("True!\n");
															else printf("False!\n");
															ObrisiListu(&$4);
															ObrisiListu(&$2);
	}
	| CARD skup ';' { printf("%d\n",BrojElemenata($2));
										ObrisiListu(&$2);
	}
	| UNIVERSE '=' skup ';' { if(universe !=NULL)
															ObrisiListu(&universe);
														else universe = $3;	}
	;
	
skup:	'~' skup 		{ if(universe == NULL)
											yyerror("Universe is not yet defined.\n");
										$$ = KopijaListe(universe);
										Razlika(&$$,$2);
										ObrisiListu(&$2);}
	| skup U skup  { $$=$1;
										Unija(&$$,$3);} 
	| skup P skup			{$$ = $1;
										Presek(&$$,$3);}
	|  skup R skup			{$$ = $1;
										Razlika(&$$,$3);}
	|'{' lista '}'			{$$ = $2;}
	| BROJ TT BROJ				{FormirajListu(&$$, $1, $3);}
	| ID									{$$ = KopijaListe(promenljive[$1-'A']);}
	;

lista : lista ',' BROJ	{ $$ = $1; 
													Dodaj(&$$,$3);}
	| BROJ								{ $$ =NULL;
													Dodaj(&$$,$1);}
	;
%%

int main(){
	Inicijalizacija();
	yydebug =0;

	yyin = fopen("ulaz.txt","r");
	if(yyin == NULL)
		yyerror("Test file can't be open!\n");
	Inicijalizacija();
	
	yyparse();
	
	Deinicijalizacija();
	
 return 0;
}